import sys

from subdomain_takeover_tools.authoritative_resolve import query_authoritative, DoesNotExist


def main():
    for line in sys.stdin:
        (subdomain, target) = line.strip().split(",")
        validate(subdomain, target)


def validate(subdomain, target):
    try:
        result = query_authoritative(subdomain, request_type='CNAME')
        if len(result) != 1:
            print("No result for %s" % subdomain)
        elif result[0] != target:
            print("Record '%s' was changed, points to %s" % (subdomain, result[0]))
    except DoesNotExist:
        print("Record '%s' was removed" % subdomain)
    except Exception as e:
        print("Validation of '%s' failed: %s" % (subdomain, e))
        return None


if __name__ == "__main__":
    main()



import sys

from subdomain_takeover_tools.confirm_agile_crm import is_valid as agile_crm_is_valid
from subdomain_takeover_tools.confirm_azure_app_service import is_valid as azure_app_service_is_valid
from subdomain_takeover_tools.confirm_azure_edge_cdn import is_valid as azure_edge_cdn_is_valid
from subdomain_takeover_tools.confirm_azure_traffic_manager import is_valid as azure_traffic_manager_is_valid
from subdomain_takeover_tools.confirm_bigcartel import is_valid as bigcartel_is_valid
from subdomain_takeover_tools.confirm_cargo import is_valid as cargo_is_valid
from subdomain_takeover_tools.confirm_elb import is_valid as elb_is_valid
from subdomain_takeover_tools.confirm_fastly import is_valid as fastly_is_valid
from subdomain_takeover_tools.confirm_github import is_valid as github_is_valid
from subdomain_takeover_tools.confirm_pantheon import is_valid as pantheon_is_valid
from subdomain_takeover_tools.confirm_s3 import is_valid as s3_is_valid
from subdomain_takeover_tools.confirm_shopify import is_valid as shopify_is_valid
from subdomain_takeover_tools.confirm_surge import is_valid as surge_is_valid
from subdomain_takeover_tools.confirm_tumblr import is_valid as tumblr_is_valid
from subdomain_takeover_tools.confirm_unclaimed import is_valid as unclaimed_is_valid


def main():
    inverse = '--inverse' in sys.argv
    strict = '--strict' in sys.argv
    full = '--full' in sys.argv

    data = sys.stdin.read()

    lines = data.strip().split('\n')
    for line in lines:
        if not line.strip():
            continue
        elif ']\t\t' not in line:
            raise IOError("Unexpected input received, currently only subtake output is supported")

        (service, target, domain) = _process_line(line)

        result = _process_subtake_output(service, target, domain, inverse)

        if result:
            if full:
                print(line)
            else:
                print(domain)


def _process_line(line):
    (parts, domain) = line.split('\t\t')
    if ': ]' in parts:
        service = parts[1:-3]
        target = ''
    else:
        (service, target) = parts[1:-2].split(': ')

    return service, target, domain


def _process_subtake_output(service, target, domain, inverse):
    result = _perform_check(service, target, domain)

    if result is None:
        return

    # xor
    return inverse != result


def _perform_check(service, target, domain):
    if service == 'agilecrm':
        return agile_crm_is_valid(domain, target)
    elif service == 'azure':
        if target.endswith('azurewebsites.net'):
            return azure_app_service_is_valid(domain, target)
        elif target.endswith('azureedge.net'):
            return azure_edge_cdn_is_valid(domain, target)
        elif target.endswith('trafficmanager.net'):
            return azure_traffic_manager_is_valid(domain, target)
        elif target.endswith('cloudapp.azure.com'):
            # for now, we assume cloudapp is vulnerable
            return True
        else:
            # other Azure services are not yet supported
            return None
    elif service == 'bigcartel':
        return bigcartel_is_valid(domain, target)
    elif service == 'cargo':
        return cargo_is_valid(domain, target)
    elif service == 'elasticbeanstalk':
        return elb_is_valid(domain, target)
    elif service == 'fastly':
        return fastly_is_valid(domain, target)
    elif service == 'github':
        return github_is_valid(domain, target)
    elif service == 'pantheon':
        return pantheon_is_valid(domain, target)
    elif service == 's3 bucket':
        return s3_is_valid(domain, target)
    elif service == 'shopify':
        return shopify_is_valid(domain, target)
    elif service == 'surge':
        return surge_is_valid(domain, target)
    elif service == 'tumblr':
        return tumblr_is_valid(domain, target)
    elif service == 'unclaimed':
        return unclaimed_is_valid(domain, target)
    else:
        return None


if __name__ == "__main__":
    main()



import sys

import dns.query
import dns.resolver


def _query_authoritative_ns(domain, log=lambda msg: None):
    default = dns.resolver.get_default_resolver()
    nameserver = default.nameservers[0]

    domain_parts = domain.split('.')
    result = None

    for i in range(len(domain_parts), 0, -1):
        sub = '.'.join(domain_parts[i - 1:])

        log('Looking up %s on %s' % (sub, nameserver))
        query = dns.message.make_query(sub, rdtype=dns.rdatatype.NS)
        response = dns.query.udp(query, nameserver, timeout=30)

        rcode = response.rcode()
        if rcode == dns.rcode.NOERROR:
            pass
        elif rcode == dns.rcode.NXDOMAIN:
            if i < len(domain_parts) - 1 and i != 1:
                sys.stderr.write(sub + " does not exist, skipping\n")
                continue
            else:
                raise DoesNotExist('%s does not exist.' % sub)
        else:
            raise Exception('Error %s' % (dns.rcode.to_text(rcode)))

        if len(response.authority) > 0:
            rrsets = response.authority
        elif len(response.additional) > 0:
            rrsets = [response.additional]
        else:
            rrsets = response.answer

        # Handle all RRsets, not just the first one
        for rrset in rrsets:
            for rr in rrset:
                if rr.rdtype == dns.rdatatype.SOA:
                    log('Same server is authoritative for %s' % sub)
                elif rr.rdtype == dns.rdatatype.A:
                    nameserver = list(rr.items)[0].address
                    log('Glue record for %s: %s' % (rr.name, nameserver))
                elif rr.rdtype == dns.rdatatype.NS:
                    authority = rr.target
                    nameserver = default.resolve(authority).rrset[0].to_text()
                    log('%s [%s] is authoritative for %s; ttl %i' %
                        (authority, nameserver, sub, rrset.ttl))
                    result = rrset
                else:
                    # IPv6 glue records etc
                    # log('Ignoring %s' % (rr))
                    pass

    return result


def query_authoritative(domain, request_type='A', log=lambda msg: None):
    authority = _query_authoritative_ns(domain, log)

    default = dns.resolver.get_default_resolver()
    nameserver = default.resolve(authority[0].target).rrset[0].to_text()
    query = dns.message.make_query(domain, request_type)
    query_result = dns.query.udp(query, nameserver, timeout=30)

    result = []
    if query_result.answer:
        for rrset in query_result.answer:
            for rr in rrset:
                result.append(str(rr))

    return result


def verbose(msg):
    sys.stderr.write(msg + u'\n')


def main():
    for s in sys.argv[1:]:
        r = query_authoritative(s, 'A', verbose)
        print()
        print('Authoritative answer:', r)


class DoesNotExist(Exception):
    pass


if __name__ == "__main__":
    main()



from subdomain_takeover_tools.helper.main import bootstrap
from subdomain_takeover_tools.helper.prepare import prepare_domain_name


def is_valid(_, cname):
    if cname is None:
        return False

    return prepare_domain_name(cname) == 'subdomain.cargocollective.com'


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import sys
import time

import dns
import requests
from tldextract import tldextract

from subdomain_takeover_tools.helper.main import bootstrap, settings
from subdomain_takeover_tools.helper.prepare import prepare_domain_name

# once we include additional support we might need to lazy initialize
transip_session = requests.Session()
transip_session.headers['Content-Type'] = 'application/json'
transip_session.headers['Authorization'] = 'Bearer ' + settings['transip']['access_token']

tld_blacklist = [
    '.cn',
    '.gov',
    '.int',
    '.mil',
    '.edu',
    '.aws',
    '.lb' # only third level is allowed
]


def is_valid(domain, cname):
    if cname is None:
        return False

    if not cname_still_valid(domain, cname):
        return False

    return confirm_unclaimed(cname)


def cname_still_valid(domain, cname):
    try:
        answers = dns.resolver.resolve(domain, "CNAME")
        for rdata in answers:
            if rdata.target.to_text(omit_final_dot=True) == cname:
                return True
    except dns.exception.DNSException:
        pass

    return False


def confirm_unclaimed(cname):
    domain_name = _extract_single_domain_name(prepare_domain_name(cname))

    for blacklisted in tld_blacklist:
        if domain_name.endswith(blacklisted):
            return False

    if settings['transip'] is not None:
        return confirm_unclaimed_transip(domain_name)
    else:
        sys.stderr.write("No supported unclaimed configuration is setup, please consult the documentation")


def confirm_unclaimed_transip(domain_name):
    time.sleep(5)
    r = transip_session.get("https://api.transip.nl/v6/domain-availability/" + domain_name)
    data = r.json()
    if 'availability' not in data:
        raise Exception("Request failed: " + r.content.decode())

    return data['availability']['status'] == 'free'


def _extract_single_domain_name(subdomain):
    r = tldextract.extract(subdomain)
    return '.'.join(r[-2:])


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import boto3

from subdomain_takeover_tools.helper.main import bootstrap
from subdomain_takeover_tools.helper.prepare import prepare_domain_name


def is_valid(_, cname):
    if cname is None:
        return False

    return confirm_elb(cname)


def confirm_elb(cname):
    cname = prepare_domain_name(cname)

    if cname.count('.') == 3:
        (prefix, region, _, _) = cname.split('.')
        # Invalid length for parameter CNAMEPrefix, value: 3, valid min length: 4
        if len(prefix) < 4 or prefix.startswith('eba-'):
            return False

        # whenever a region contains '-gov-' we will no be able to take it over
        if '-gov-' in region:
            return False

        client = boto3.client('elasticbeanstalk', region_name=region)
        result = client.check_dns_availability(CNAMEPrefix=prefix)

        if result['Available']:
            return True
    return False


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import requests
from azure.identity import DefaultAzureCredential

from subdomain_takeover_tools.helper.main import bootstrap
from subdomain_takeover_tools.helper.prepare import resolve_cname

EDGE_CDN = '.azureedge.net'

credential = DefaultAzureCredential()

session = requests.Session()
(token, _) = credential.get_token('https://management.azure.com/.default')
session.headers['Authorization'] = "Bearer " + token
url = "https://management.azure.com/providers/Microsoft.Cdn/checkNameAvailability?api-version=2019-12-31"


def is_valid(hostname, cname):
    if hostname == cname:
        cname = resolve_cname(hostname)

    if cname is None:
        return False

    return confirm_azure_edge_cdn(cname)


def confirm_azure_edge_cdn(cname):
    try:
        if cname.count('.') == 2 and EDGE_CDN in cname:
            dns_prefix = cname.replace(EDGE_CDN, '')
            result = session.post(url, json={'name': dns_prefix, 'type': 'Microsoft.Cdn/Profiles/Endpoints'})
            return result.json()['nameAvailable']
    except KeyError:
        pass
    return False


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



from unittest import TestCase

from subdomain_takeover_tools.extract_domain_names import extract_domain_name


class Test(TestCase):
    def test_extract_domain_name(self):
        self.assertEqual(
            "github.com",
            extract_domain_name("test.github.com")
        )

    def test_extract_domain_name_co_uk(self):
        self.assertEqual(
            "github.co.uk",
            extract_domain_name("test.test.github.co.uk")
        )

    def test_extract_domain_names(self):
        self.assertEqual(
            "github.com\ngithub.io",
            extract_domain_name("*.github.(com|io)")
        )

    def test_brackets_in_the_front(self):
        self.assertEqual(
            "endlessgroup.org",
            extract_domain_name("(*).endlessgroup.org")
        )

    def test_brackets_in_the_front_2(self):
        self.assertEqual(
            "sharefile.com",
            extract_domain_name("(yoursubdomain).sharefile.com/sf/v3/")
        )



import requests
import urllib3

from subdomain_takeover_tools.helper.main import bootstrap

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

ONLY_ONE_STEP_LEFT = 'Only one step left!'

headers = {
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.71 Safari/537.36"
}


def is_valid(hostname, _):
    return confirm_shopify(hostname)


def confirm_shopify(hostname):
    if hostname.endswith('.myshopify.com') or hostname.endswith('.shopify.com'):
        return False

    try:
        r = requests.get('http://%s' % hostname, verify=False, timeout=15, headers=headers)
        return ONLY_ONE_STEP_LEFT in r.text
    except (requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout):
        return False


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()
